#!/usr/physics/gauche/bin/gosh

(use srfi-1)
(use srfi-13)
(use srfi-19)
(use text.html-lite)
(use text.tree)
(use www.cgi)
(use gauche.collection)
(use gauche.process)

(use dbi)
(use dbd.pg)

(define DEFAULT-PAST-HOURS 12)
(define MAX-HOURS 720)  ; some arbitrary max to clamp down on craziness
(define ROOM-NUMBER 111)
(define DEVICE-NAME "gizmo")

(define (get-integer-param name params)
  (let ((p (cgi-get-parameter name params)))
    (if p (string->number p) #f)
  )
)

(define (start-gnuplot)
  (run-process '( "/usr/physics/gnuplot/bin/gnuplot" ) :input :pipe)
  ;(run-process '( "cat" ) :input :pipe :output "blah.gnuplot")
)

(define (main args)
  (let* ((pg-conn (dbi-make-connection (dbi-make-driver "pg")
                    "username" "password" "dbname=templog2"))
         (params (cgi-parse-parameters)))

    (let ((hours (get-integer-param "hours" params))
          (room  (get-integer-param "room" params))
          (dev   (cgi-get-parameter "device" params))
          (pp    (start-gnuplot)))

      (write-tree (cgi-header :content-type "image/png"))
      (flush-all-ports) ; this is needed to make sure the header is
                        ; output first.

      (plot-device-data
        pg-conn
        (if room room ROOM-NUMBER)
        (if dev dev DEVICE-NAME)
        (make-stamp-string (if hours hours DEFAULT-PAST-HOURS))
        (process-input pp))
      (process-wait pp)
    )
    (dbi-close pg-conn)
  )
)

(define (make-past hours)
  (time-utc->date
    (make-time
      time-utc
      0
      (- (time-second (current-time)) (* hours 3600))))
)

(define (make-stamp-string hours)
  (if (or (not (number? hours)) (< hours 1))
    (make-stamp-string 1)
    (let ((date (make-past hours)))
      (format #f "~a-~a-~a ~a:~a:~a"
        (date-year date) (date-month date)
        (date-day date) (date-hour date)
        (date-minute date) (date-second date))
    )
  )
)

(define (cvt-date stamp-string)
  (date->string
    (string->date stamp-string "~Y-~m-~d ~H:~M:~S")
    "~Y~m~d~H~M~S")
)

(define (do-sql conn sql . args)
  (if (null? args)
    (dbi-execute (dbi-prepare conn sql :pass-through #t))
    (dbi-execute
      (dbi-prepare conn
        (apply format #f sql
          (map (lambda (a) (if (string? a) (dbi-escape-sql conn a) a)) args))
        :pass-through #t))
  )
)

(define (get-sensor-list pg-conn deviceid)
  (map
    (lambda (row) row)
    (do-sql pg-conn
      "SELECT sensorid,sensorname,description FROM sensor WHERE deviceid='~a'"
      deviceid)
  )
)

(define gnuplot-boiler-plate
'(
"set terminal png\n"
"#set output \"temp.png\"\n"
"set key below\n"
"set xdata time\n"
"set xlabel \" \"\n"
"set timefmt \"%Y%m%d%H%M%S\"\n"
))

(define (plot-device-data pg-conn room devicename time-constraint plot-inp)
  (for-each
    (lambda (row)
      (for-each (lambda (cmd) (display cmd plot-inp)) gnuplot-boiler-plate)
      (format plot-inp "plot ~a\n"
        (string-join (map (lambda (srow)
        (format #f "\"-\" using 1:2 with lines title \"~a\""
          (dbi-get-value srow 2)))
           (get-sensor-list pg-conn (dbi-get-value row 0))) ","))
      (for-each
        (lambda (srow)
          (get-sensor-data pg-conn (dbi-get-value srow 0)
            time-constraint plot-inp)
          (display "e\n" plot-inp)
        )
        (get-sensor-list pg-conn (dbi-get-value row 0))
      )
    )
    (do-sql pg-conn
      "SELECT deviceid FROM device WHERE room=~a and devicename='~a'"
      room devicename)
  )
  (display "exit\n" plot-inp)
)

(define (get-sensor-data pg-conn sensorid time-constraint plot-inp)
  (for-each
    (lambda (row)
      (format plot-inp "~a ~a\n"
        (cvt-date (dbi-get-value row 1)) (dbi-get-value row 0))
    )
    (do-sql pg-conn
      "SELECT sensordata.value,sensordata.stamp FROM sensordata WHERE sensordata.sensorid=~a AND sensordata.stamp > '~a' ORDER BY sensordata.stamp"
      sensorid time-constraint)
  )
)
