#!/usr/physics/gauche/bin/gosh

(use srfi-1)
(use srfi-19)
(use text.html-lite)
(use text.tree)
(use www.cgi)
(use gauche.collection)

(use dbi)
(use dbd.pg)

(define DEFAULT-PAST-HOURS 12)
(define MAX-HOURS 720)  ; some arbitrary max to clamp down on craziness
(define ROOM-NUMBER 111)
(define DEVICE-NAME "gizmo")

(define (get-integer-param name params)
  (let ((p (cgi-get-parameter name params)))
    (if p (string->number p) #f)
  )
)

(define (main args)
  (let* ((pg-conn (dbi-make-connection (dbi-make-driver "pg")
                    "username" "password" "dbname=templog2"))
         (params (cgi-parse-parameters)))

    (let ((hours (get-integer-param "hours" params))
          (room  (get-integer-param "room" params))
          (dev   (cgi-get-parameter "device" params)))
      (write-tree (cgi-header :content-type "text/plain"))

      (get-device-data
        pg-conn
        (if room room ROOM-NUMBER)
        (if dev dev DEVICE-NAME)
        (make-stamp-string (if hours hours DEFAULT-PAST-HOURS)))
    )

    (dbi-close pg-conn)
  )
)

(define (simple pg-query params)
  (let ((hours-input (cgi-get-parameter "hours" params)))
    (if hours-input
      (let ((hours (string->number hours-input)))
        (emit-text pg-query (if hours hours DEFAULT-PAST-HOURS))
      )
      (emit-text pg-query DEFAULT-PAST-HOURS)
    )
  )
)

(define (make-past hours)
  (time-utc->date
    (make-time
      time-utc
      0
      (- (time-second (current-time)) (* hours 3600))))
)

(define (make-stamp-string hours)
  (let ((date (make-past hours)))
    (format #f "~a-~a-~a ~a:~a:~a"
      (date-year date) (date-month date)
      (date-day date) (date-hour date)
      (date-minute date) (date-second date))
  )
)

(define (emit-text pg-query hours)
  (write-tree (cgi-header :content-type "text/plain"))
  (for-each
    (lambda (row)
      (print (dbi-get-value row 0) " \t"
        (dbi-get-value row 1) " \t" (dbi-get-value row 2))
    )
    (dbi-execute-query pg-query (make-sql hours))
  )
)

(define (emit-html pg-query)
  (write-tree (cgi-header))
  (write-tree (html:html (html:body (emit-html-table pg-query))))
)

(define (emit-html-table pg-query)
  (html:table :width "100%"
    (html:tr :bgcolor "#8fff8f"
      (html:td "Stamp")
      (html:td "Sensors ID")
      (html:td "Value")
    )
    (map
      (lambda (row)
        (html:tr
          (html:td (dbi-get-value row 0))
          (html:td (dbi-get-value row 1))
          (html:td (dbi-get-value row 3)))
      )
      (dbi-execute-query pg-query (make-sql hours))
    )
  )
)

(define (do-sql conn sql . args)
  (if (null? args)
    (dbi-execute (dbi-prepare conn sql :pass-through #t))
    (dbi-execute
      (dbi-prepare conn
        (apply format #f sql
          (map (lambda (a) (if (string? a) (dbi-escape-sql conn a) a)) args))
        :pass-through #t))
  )
)

(define (get-sensor-list pg-conn deviceid)
  (map
    (lambda (row) row)
    (do-sql pg-conn
      "SELECT sensorid,sensorname,description FROM sensor WHERE deviceid='~a'"
      deviceid)
  )
)

(define (get-device-list pg-conn room devicename time-constraint)
  (for-each
    (lambda (row)
      (for-each
        (lambda (srow)
          (print (dbi-get-value srow 1) " , " (dbi-get-value srow 2))
          (get-sensor-data pg-conn (dbi-get-value srow 0) time-constraint)
        )
        (get-sensor-list pg-conn (dbi-get-value row 0))
      )
    )
    (do-sql pg-conn
      "SELECT deviceid FROM device WHERE room=~a and devicename='~a'"
      room devicename)
  )
)

(define (get-sensor-data pg-conn sensorid time-constraint)
  (for-each
    (lambda (row)
      (print (dbi-get-value row 0) "," (dbi-get-value row 1))
    )
    (do-sql pg-conn
      "SELECT sensordata.value,sensordata.stamp FROM sensordata WHERE sensordata.sensorid=~a AND sensordata.stamp > '~a' ORDER BY sensordata.stamp"
      sensorid time-constraint)
  )
)

(define (get-device-data pg-conn room devicename time-constraint)
  (get-device-list pg-conn room devicename time-constraint)
)
