#!/usr/physics/gauche/bin/gosh

(use srfi-1)
(use gauche.net)
(use gauche.uvector)
(use file.util)
(use srfi-19)
(use gauche.collection)

(use dbi)
(use dbd.pg)

;
; Some constants to peruse.
;
(define-constant packetlen 14)
(define debug #f)

(define (do-sql conn sql . args)
  (if (null? args)
    (dbi-execute (dbi-prepare conn sql :pass-through #t))
    (dbi-execute
      (dbi-prepare conn
        (apply format #f sql
          (map (lambda (a) (if (string? a) (dbi-escape-sql conn a) a)) args))
        :pass-through #t))
  )
)

(define (make-stamp-string)
  (let ((date (current-date)))
    (format #f "~a-~a-~a ~a:~a:~a"
      (date-year date) (date-month date)
      (date-day date) (date-hour date)
      (date-minute date) (date-second date))
  )
)

(define INSERT-DATA "INSERT INTO sensordata ( sensorid, stamp, value ) VALUES ( ~a, '~a', ~a.~a )")

;
; Put a packet in the log file.
;
; Probably should wrap this stuff up and do some exception handling.
;
(define (packet-output vec pg-conn bldg-sid room-sid physics-sid)
  (guard
    (e
      (else
        (do-sql pg-conn "ROLLBACK")
        (print "Query failed: " (slot-ref e 'message))
        (flush-all-ports) ; Gauche-ism
      )
    )
    (let ((stamp-string (make-stamp-string)))
      (do-sql pg-conn "BEGIN")
      ; buildingvent
      (do-sql pg-conn INSERT-DATA
        bldg-sid stamp-string (u8vector-ref vec 1) (u8vector-ref vec 2))
      ; physicsvent
      (do-sql pg-conn INSERT-DATA
        physics-sid stamp-string (u8vector-ref vec 3) (u8vector-ref vec 4))
      ; roomtemp
      (do-sql pg-conn INSERT-DATA
        room-sid stamp-string (u8vector-ref vec 5) (u8vector-ref vec 6))
      (do-sql pg-conn "COMMIT")
    )
  )
)

(define (good-data-p? vec data-index)
  (cond
    ((< data-index 1) (error "Bad index to good-data? " data-index))
    ((> data-index 6) #t)
    ((> (u8vector-ref vec data-index) 99)
       (print "Weird data: " (u8vector-ref vec data-index))
       #f)
    (else (good-data-p? vec (+ data-index 1)))
  )
)
(define (good-data? vec) (good-data-p? vec 1))
   
;
; This recurses to read the socket over and over ...
;
(define (sock-process vec port pg-conn bldg-sid room-sid physics-sid)
  (read-block! vec port)

  (if (good-data? vec)
    (packet-output vec pg-conn bldg-sid room-sid physics-sid)
    (print "Bad data not logged: " vec)
  )

  (sock-process vec port pg-conn bldg-sid room-sid physics-sid)
)

(define (sharm pg-conn devicename room sensorname)
  (map
    (lambda (row) (dbi-get-value row 0))
    (do-sql pg-conn
      "SELECT sensorid FROM sensor,device WHERE sensor.sensorname='~a' AND device.devicename='~a' AND device.deviceid=sensor.deviceid AND device.room=~a"
      sensorname devicename room)
  )
)

(define (get-sensorid pg-conn devicename room sensorname)
  (let ((result (sharm pg-conn devicename room sensorname)))
    (cond
      ((> (length result) 1)
       (error "Too many sensor IDs returned for sensorname: " sensorname))
      ((< (length result) 1)
       (error "No sensor ID returned for sensorname: " sensorname))
      (else (first result))
    )
  )
)

(define (get-socket-input-port)
  (let ((sock (make-socket PF_INET SOCK_DGRAM)))
    (socket-bind sock (make <sockaddr-in> :port 1950))
    (let ((inp (socket-input-port sock)))
      ((setter port-buffering) inp :full)
      inp
    )
  )
)

;
; Initialize some stuff.  Invoke the socket processor.
;
(define (main args)
  (let ((pg-conn (dbi-make-connection (dbi-make-driver "pg")
                   "username" "password" "dbname=templog2")))

    (let ((bldg-sid (get-sensorid pg-conn "gizmo" 111 "buildingvent"))
          (room-sid (get-sensorid pg-conn "gizmo" 111 "roomtemp"))
          (physics-sid (get-sensorid pg-conn "gizmo" 111 "physicsvent"))
          (vec (make-u8vector packetlen))
          (inp (get-socket-input-port)))

        (if debug
          (sock-process vec inp pg-conn bldg-sid room-sid physics-sid)
          (serverize
            "udpdaemonlogfile2.txt"
            (lambda ()
              (sock-process vec inp pg-conn bldg-sid room-sid physics-sid)))
        )
    )

    (dbi-close pg-conn)
  )
)

(define (serverize log-filename proc)
  (let ((rv (sys-fork)))
    (cond
      ((= rv 0) (sys-setsid))
      ((> rv 0) (exit 0))
      (else (print "fork failed"))
    )
  )

  (close-input-port (standard-input-port))
  (close-output-port (standard-error-port))
  (close-output-port (standard-output-port))

  (if log-filename
    (let ((op (open-output-file log-filename)))
      (with-ports #f op op proc)
    )
    (if proc (proc))
  )
)
